'''
Created on Feb 2, 2012

@author: mistaguy
-This file must contain any database connection wanted
'''

import MySQLdb
import psycopg2
from _mysql_exceptions import MySQLError
class Connect:
    """This class for creating a connection to the database engine
       This the Core SQL Engine Class   
    """     
    server = "127.0.0.1"
    database = "eshule"
    username = "root"
    password = "root"
    portnumber = 3306
    dbengine="MySQL"
    items={}
    def __init__(self):
        """
        Optionally pass in an initial dictionary of connection items
        """                 
           
        return       
    def __set_connection__(self):
        self.items["server"] =self.server
                
        self.items["database"] = self.database
               
        self.items["username"] = self.username
                
        self.items["dbengine"] = self.dbengine
                  
        self.items["password"] = self.password 
        
    def filling(self, sqlstr):
        """
        Gets a dataset
        """
        #set connection dictionary keys
        self.__set_connection__()
        
        # Open database connection
        
        if self.dbengine=="MySQL":
            try:
                connection=MySQLdb.connect(host=self.items['server'],db=self.items['database'],user=self.items['username'],passwd=self.items['password'])
     
                # prepare a cursor object using cursor() method
                cursor =connection.cursor()
            
                # Execute the SQL command
                cursor.execute(sqlstr)
                # Fetch all the rows in a list of lists.
                results = cursor.fetchall()
                return results
            except:            
                MySQLError
                return None
            connection.close()
        elif self.dbengine=="PSQL":
            try:
                connection=psycopg2.connect(host=self.items['server'],database=self.items['database'],user=self.items['username'],password=self.items['password'])
     
                # prepare a cursor object using cursor() method
                cursor =connection.cursor()
              
                # Execute the SQL command
                cursor.execute(sqlstr)
                # Fetch all the rows in a list of lists.
                results = cursor.fetchall()
                return results
            except psycopg2.Error,e:   
                print "Error %s" % (e)
                return None
                connection.close()
                
        elif self.dbengine=="Oracle":
            try:
                connection=MySQLdb.connect(host=self.items['server'],db=self.items['database'],user=self.items['username'],passwd=self.items['password'])
     
                # prepare a cursor object using cursor() method
                cursor =connection.cursor()
          
                # Execute the SQL command
                cursor.execute(sqlstr)
                # Fetch all the rows in a list of lists.
                results = cursor.fetchall()
                return results
            except psycopg2.Error,e:           
                print "Error %d: %s" % (e.args[0], e.args[1])
                return None
            connection.close()
            
        
        
        

    def nonequery(self, sqlstr):
        """
        Executes Queries that do not return Datasets
        """
        #set connection dictionary keys
        self.__set_connection__()
        if self.dbengine=="MySQL":
            try:
                # Open database connection
                connection=MySQLdb.connect(host=self.items['server'],db=self.items['database'],user=self.items['username'],passwd=self.items['password'])
                # prepare a cursor object using cursor() method
                cursor =connection.cursor()
         
                # Execute the SQL command
                cursor.execute(sqlstr)
                # Commit your changes in the database
                connection.commit()
                return True
            except MySQLdb.Error, e:            
                MySQLError
                #Roll Back incase of failure
                #connection.roleback()
                print "Error %d: %s" % (e.args[0], e.args[1])
                return False
            connection.close() 
        elif self.dbengine=="PSQL":
            try:
                # Open database connection
                connection=psycopg2.connect(host=self.items['server'],database=self.items['database'],user=self.items['username'],password=self.items['password'])
                # prepare a cursor object using cursor() method
                cursor =connection.cursor()
            
                # Execute the SQL command
                cursor.execute(sqlstr)
                # Commit your changes in the database
                connection.commit()
                return True
            except psycopg2.Error,e:           
                print "Error %s" % (e)
                return None
            connection.close()
            
        elif self.dbengine=="Oracle":
            try:
                # Open database connection
                connection=MySQLdb.connect(host=self.items['server'],db=self.items['database'],user=self.items['username'],passwd=self.items['password'])
                # prepare a cursor object using cursor() method
                cursor =connection.cursor()            
                # Execute the SQL command
                cursor.execute(sqlstr)
                # Commit your changes in the database
                connection.commit()
                return True
            except:            
                MySQLError
                #Roll Back incase of failure
                #connection.roleback()
                print "Error has occured"
                return False
            connection.close()
            
       

    def scalar(self, sqlstr):
        """
        return a Dataset
        """
        #set connection dictionary keys
        self.__set_connection__()
        if self.dbengine=="MySQL":
            try:
                # Open database connection
                connection=MySQLdb.connect(host=self.items['server'],db=self.items['database'],user=self.items['username'],passwd=self.items['password'])
                # prepare a cursor object using cursor() method
                cursor =connection.cursor()
            
                # Execute the SQL command
                cursor.execute(sqlstr)
                # Fetch a single row using fetchone() method.
                result = cursor.fetchone()
                return result
            except:            
                MySQLError            
                return None
            connection.close()
        elif self.dbengine=="PSQL":
            try:
                # Open database connection
                connection=psycopg2.connect(host=self.items['server'],dbname=self.items['database'],user=self.items['username'],password=self.items['password'])
                # prepare a cursor object using cursor() method
                cursor =connection.cursor()
            
                # Execute the SQL command
                cursor.execute(sqlstr)
                # Fetch a single row using fetchone() method.
                result = cursor.fetchone()
                return result
            except psycopg2.Error,e:           
                print "Error %s" % (e)        
                          
                return None
        
        elif self.dbengine=="Oracle":
            try:
                # Open database connection
                connection=MySQLdb.connect(host=self.items['server'],db=self.items['database'],user=self.items['username'],passwd=self.items['password'])
                # prepare a cursor object using cursor() method
                cursor =connection.cursor()
            
                # Execute the SQL command
                cursor.execute(sqlstr)
                # Fetch a single row using fetchone() method.
                result = cursor.fetchone()
                return result
            except:           
                           
                return None
            connection.close()


'''
Created on Feb 2, 2012

@author: mistaguy

@summary: uses the connect class to provide a database abstract layer to tables

'''

from Connect import Connect
import MySQLdb
import psycopg2
class ConnectOperations:
    """
    This Class is the base CRUD class for all database tables models.
    It use an instance of the Connect class    """
    
    def __init__(self):
        """
        constructor
        """        
        return      
    con=Connect()
    def insert(self,obj):
        """
        Creates a new Record to the database
        """
        #valid fields
        if not obj["table"]:
            print "Missing Table"
            return
        try:
            
            print "Attempting to Populate Records"
            attributes=""
            values=""
            table=obj["table"]
            for o in obj:
                if o=="clause" or o=="table":
                    print ""
                else:
                    attributes=o+","+attributes
                    values=obj[o]+","+values 
            #remove the last comma    
            attributes=attributes[0:len(attributes)-1] 
            values=values[0:len(values)-1]       
            sqlstr="INSERT INTO " + table + "(" + attributes + ")VALUES(" + values + ")"       
            result= self.con.nonequery(sqlstr)
            print "Record Inserted"
            return result
        except MySQLdb.Error or psycopg2.Error, e:
            if self.con.dbengine=="MySQL":
                print e.message
            elif self.con.dbengine=="PSQL":
                print e.message
            print "may be make sure your dictionary values corresponds to the table in your database"
            
            return
    
    
    def update(self, obj):        
        """
        modify a record.        
        """
        if not obj["table"]:
            print "Missing Table"
            return
        try:
            if not obj.has_key("clause"): 
                obj["clause"]=" "
            print "Attempting to Modify a Record"
            statement = ""
            for o in obj:
                if o=="clause" or o=="table":
                    print ""
                else:
                    statement= (o+"="+obj[o]+",")+statement
                
            #remove the last comma    
            statement=statement[0:len(statement)-1] 
            sqlstr="UPDATE " + obj["table"] + " SET "+statement +" "+obj["clause"]
            result=self.con.nonequery(sqlstr)
            
            print "Record Updated"
            return result
    
        except:
            print "may be make sure your dictionary values corresponds to the table in your database"
            
            return
    
    def select_dataset(self,obj ):
        """
        returns a dataset OR datasets
        """           

        print "Attempting to select record"
     
        if not obj.has_key("clause"): 
            obj["clause"]=" "
        if not obj.has_key("attributes"): 
            obj["attributes"]="*"
        try:
            
            
            sqlstr="SELECT " + obj["attributes"] + " FROM " + obj["table"] + " " + obj["clause"]
            
            result=self.con.filling(sqlstr)
            print "Records Selected"
            return result
        except MySQLdb.Error,e:
            if self.con.dbengine=="MySQL":
                print e.message
            
            print "may be make sure your dictionary values corresponds to the table in your database"
            
            return
    ''' def select_single_dataset(self, obj):
        """
       return one dataset
        """  
        try:   
            if not obj.has_key("clause"):                 
                print "You must provide a 'clause' dictionary key with values eg where id =1"
                return
            if not obj.has_key("attribute"):                 
                print "You must provide an 'attribute' dictionary key with values eg id"
                return
            print "attempting to select single record"
            sqlstr=Connect().scalar("SELECT " + obj["attribute"] + " FROM " + obj["table"] + " " + obj["clause"])
            print sqlstr
            result=sqlstr
            print "single record selected"
            return result
        except:
            print "may be make sure your dictionary values corresponds to the table in your database"
            
            return
    '''
    def delete(self, obj):
        """
        remove a record       
        """   
        try: 
            if not obj.has_key("clause"): 
                print "You must provide a where clause dictionary key with values eg where id =1"
              
                return
            print "Attempting to delete a record or records"  
            result=self.con.nonequery("DELETE FROM " +obj["table"]+ " " + obj["clause"])
            print "single record(s) deleted"              
            return result
        except:
            print "may be make sure your dictionary values corresponds to the table in your database"
            
            return


from distutils.core import setup

setup(
    name='greenapple',
    version='0.1.2',
    author='Acellam Guy',
    author_email='abiccel@yahoo.com',
    packages=['greenapple','greenapple.model'],    
    url='http://pypi.python.org/pypi/greenapple/',
    license='LICENSE.txt',
    description='Useful Database Layer Package',
    long_description=open('README.txt').read(),
    #install_requires=[
    #    "Django >= 1.1.1",
    #    "caldav == 0.1.4",
    #],
)
